@@ -29,6 +29,7 @@
 
 #include "clutter/clutter-device-manager-private.h"
 #include "clutter-private.h"
+#include "clutter-evdev.h"
 
 #include "clutter-input-device-evdev.h"
 
@@ -189,12 +190,55 @@ ClutterInputDeviceType
 _clutter_input_device_evdev_determine_type (struct libinput_device *ldev)
 {
 
-  if (libinput_device_has_capability (ldev, LIBINPUT_DEVICE_CAP_KEYBOARD))
-    return CLUTTER_KEYBOARD_DEVICE;
-  else if (libinput_device_has_capability (ldev, LIBINPUT_DEVICE_CAP_POINTER))
+  if (libinput_device_has_capability (ldev, LIBINPUT_DEVICE_CAP_POINTER))
     return CLUTTER_POINTER_DEVICE;
   else if (libinput_device_has_capability (ldev, LIBINPUT_DEVICE_CAP_TOUCH))
     return CLUTTER_TOUCHSCREEN_DEVICE;
+  else if (libinput_device_has_capability (ldev, LIBINPUT_DEVICE_CAP_KEYBOARD))
+    return CLUTTER_KEYBOARD_DEVICE;
   else
     return CLUTTER_EXTENSION_DEVICE;
 }
+
+/**
+ * clutter_evdev_input_device_get_libinput_device:
+ * @device: a #ClutterInputDevice
+ *
+ * Retrieves the libinput_device struct held in @device.
+ *
+ * Returns: The libinput_device struct
+ *
+ * Since: 1.20
+ * Stability: unstable
+ **/
+struct libinput_device *
+clutter_evdev_input_device_get_libinput_device (ClutterInputDevice *device)
+{
+  ClutterInputDeviceEvdev *device_evdev;
+
+  g_return_val_if_fail (CLUTTER_IS_INPUT_DEVICE_EVDEV (device), NULL);
+
+  device_evdev = CLUTTER_INPUT_DEVICE_EVDEV (device);
+
+  return device_evdev->libinput_device;
+}
+
+/**
+ * clutter_evdev_event_sequence_get_slot:
+ * @sequence: a #ClutterEventSequence
+ *
+ * Retrieves the touch slot triggered by this @sequence
+ *
+ * Returns: the libinput touch slot.
+ *
+ * Since: 1.20
+ * Stability: unstable
+ **/
+gint32
+clutter_evdev_event_sequence_get_slot (const ClutterEventSequence *sequence)
+{
+  if (!sequence)
+    return -1;
+
+  return GPOINTER_TO_INT (sequence) - 1;
+}
